@@ -1,19 +1,21 @@
 # -*- coding: utf-8 -*-
 
-version = '0.3.8'
-version_info = (0, 3, 8)
+version = '0.4.12'
+version_info = (0, 4, 12)
 
-from .utils import is_py3
+from .utils import is_py3, set_logger
 from .loop import instance, current
 from .event import EventEmitter
 from . import tcp
 from . import udp
+from . import pipe
 from .buffer import Buffer
 from .dns import DNSResolver
 from . import errors
 
 if is_py3:
     from .coroutines.future import Future
+    from .coroutines.chain import Chain
     from . import loop
 
 
