@@ -2,6 +2,7 @@
 # 2020/7/10
 # create by: snower
 
+import sys
 import time
 import struct
 import random
@@ -9,22 +10,13 @@ import traceback
 import logging
 import argparse
 from collections import deque
+import threading
 import socket
 import sevent
+from .utils import create_server, create_socket, config_signal, format_data_len, is_subnet
 
 BYTES_MAP = {"B": 1, "K": 1024, "M": 1024*1024, "G": 1024*1024*1024, "T": 1024*1024*1024*1024}
 
-def format_data_len(date_len):
-    if date_len < 1024:
-        return "%dB" % date_len
-    elif date_len < 1024*1024:
-        return "%.3fK" % (date_len/1024.0)
-    elif date_len < 1024*1024*1024:
-        return "%.3fM" % (date_len/(1024.0*1024.0))
-    elif date_len < 1024*1024*1024*1024:
-        return "%.3fG" % (date_len/(1024.0*1024.0*1024.0))
-    return "%.3fT" % (date_len/(1024.0*1024.0*1024.0*1024.0))
-
 def host_parse(host):
     hosts, subnet, cs = [], ["", 0], []
     is_brackets, is_subnet = False, False
@@ -87,18 +79,6 @@ def host_parse(host):
         subnet[1] = (~ (0xffffffffffffffff >> min(subnet[1], 64)), ~ (0xffffffffffffffff >> max(subnet[1] - 64, 0)))
     return hosts, subnet
 
-def is_subnet(ip, subnet):
-    try:
-        ip = struct.unpack("!I", socket.inet_pton(socket.AF_INET, ip))[0]
-        if isinstance(subnet[0], tuple) or isinstance(subnet[1], tuple):
-            return False
-        return (ip & subnet[1]) == (subnet[0] & subnet[1])
-    except:
-        ip = (struct.unpack("!QQ", socket.inet_pton(socket.AF_INET6, ip)))
-        if not isinstance(subnet[0], tuple) or len(subnet[0]) != 2 or not isinstance(subnet[1], tuple) or len(subnet[1]) != 2:
-            return False
-        return ((ip[0] & subnet[1][0]) == (subnet[0][0] & subnet[1][0])) and ((ip[1] & subnet[1][1]) == (subnet[0][1] & subnet[1][1]))
-
 def warp_write(conn, status, key):
     origin_write = conn.write
     def _(data):
@@ -276,8 +256,7 @@ def warp_mirror_write(mirror_host, mirror_header, warp_write_func):
                     if up_address == down_address and status.get("mirror_down_conn"):
                         status['mirror_up_conn'] = status.get("mirror_down_conn")
                     else:
-                        mirror_conn = sevent.tcp.Socket()
-                        mirror_conn.enable_nodelay()
+                        mirror_conn = create_socket(up_address)
                         mirror_conn.connect(up_address, 5)
                         mirror_conn.on_connect(lambda s: mirror_conn.end() if conn._state == sevent.tcp.STATE_CLOSED else None)
                         mirror_conn.on_data(lambda s, b: b.read())
@@ -306,8 +285,7 @@ def warp_mirror_write(mirror_host, mirror_header, warp_write_func):
                     if up_address == down_address and status.get("mirror_up_conn"):
                         status['mirror_down_conn'] = status.get("mirror_up_conn")
                     else:
-                        mirror_conn = sevent.tcp.Socket()
-                        mirror_conn.enable_nodelay()
+                        mirror_conn = create_socket(down_address)
                         mirror_conn.connect(down_address, 5)
                         mirror_conn.on_connect(lambda s: mirror_conn.end() if conn._state == sevent.tcp.STATE_CLOSED else None)
                         mirror_conn.on_data(lambda s, b: b.read())
@@ -360,8 +338,7 @@ async def tcp_forward(conns, conn, forward_address, subnet, status):
 
     try:
         conn.enable_nodelay()
-        pconn = sevent.tcp.Socket()
-        pconn.enable_nodelay()
+        pconn = create_socket(forward_address)
         await pconn.connectof(forward_address)
         pconn.write = warp_write(pconn, status, "send_len")
         logging.info("tcp forward connected %s:%d -> %s:%d", conn.address[0], conn.address[1], forward_address[0], forward_address[1])
@@ -400,25 +377,28 @@ async def tcp_forward_server(conns, server, forward_hosts, speed_limiter):
         conns[id(conn)] = (conn, status)
 
 async def check_timeout(conns, timeout):
-    while True:
-        if timeout <= 0:
-            await sevent.current().sleep(7200)
-            continue
+    def run_check():
+        while True:
+            try:
+                now = time.time()
+                for conn_id, (conn, status) in list(conns.items()):
+                    if status['check_recv_len'] != status['recv_len'] or status['check_send_len'] != status['send_len']:
+                        status["check_recv_len"] = status["recv_len"]
+                        status["check_send_len"] = status["send_len"]
+                        status['last_time'] = now
+                        continue
 
-        try:
-            now = time.time()
-            for conn_id, (conn, status) in list(conns.items()):
-                if status['check_recv_len'] != status['recv_len'] or status['check_send_len'] != status['send_len']:
-                    status["check_recv_len"] = status["recv_len"]
-                    status["check_send_len"] = status["send_len"]
-                    status['last_time'] = now
-                    continue
+                    if now - status['last_time'] >= timeout:
+                        sevent.current().add_async_safe(conn.close)
+                        conns.pop(conn_id, None)
+            finally:
+                time.sleep(min(float(timeout) / 2.0, 30))
 
-                if now - status['last_time'] >= timeout:
-                    conn.close()
-                    conns.pop(conn_id, None)
-        finally:
-            await sevent.current().sleep(30)
+    if timeout > 0:
+        check_thread = threading.Thread(target=run_check)
+        check_thread.setDaemon(True)
+        check_thread.start()
+    await sevent.Future()
 
 class Delayer(object):
     def __init__(self, delay, rdelays):
@@ -503,9 +483,8 @@ async def tcp_forward_servers(servers, timeout, speed, global_speed):
         sevent.current().call_async(tcp_forward_server, conns, server, forward_hosts, speed_limiter)
     await check_timeout(conns, timeout)
 
-if __name__ == '__main__':
-    logging.basicConfig(level=logging.INFO, format='%(asctime)s %(levelname)1.1s %(message)s',
-                    datefmt='%Y-%m-%d %H:%M:%S', filemode='a+')
+def main(argv):
+    global warp_write
 
     parser = argparse.ArgumentParser(description="tcp port forward")
     parser.add_argument('-L', dest='forwards', default=[], action="append", type=str,
@@ -521,8 +500,8 @@ if __name__ == '__main__':
                         help='mirror host, accept format [[up_host:]up_port:[down_host]:down_port] (example: 0.0.0.0:80:127.0.0.1:8088 or :127.0.0.1:8088 or 127.0.0.1:8088: or 8088:8088)')
     parser.add_argument('-F', dest='mirror_header', default="", type=str,
                         help='mirror header, accept variables [from_host|from_port|to_host|to_port|conn_id] (example: "{conn_id}-{from_host}:{from_port}->{to_host}:{to_port}\\r\\n")')
-    args = parser.parse_args()
-
+    args = parser.parse_args(args=argv)
+    config_signal()
     if not args.forwards:
         exit(0)
 
@@ -548,20 +527,20 @@ if __name__ == '__main__':
 
     forward_servers = []
     for bind_address, forward_hosts in forwards.items():
-        server = sevent.tcp.Server()
-        server.enable_reuseaddr()
-        server.listen(bind_address)
+        server = create_server(bind_address)
         forward_servers.append((server, sorted(forward_hosts,
                                                key=lambda x: x[1][1][0] * 0xffffffffffffffff + x[1][1][1] if isinstance(x[1][1], tuple) else x[1][1],
                                                reverse=True)))
         logging.info("port forward listen %s:%s", bind_address[0], bind_address[1])
 
+    sevent.current().call_async(tcp_forward_servers, forward_servers, args.timeout,
+                                int(args.speed / 10), int(args.global_speed / 10))
+
+if __name__ == '__main__':
+    logging.basicConfig(level=logging.INFO, format='%(asctime)s %(levelname)1.1s %(message)s',
+                        datefmt='%Y-%m-%d %H:%M:%S', filemode='a+')
     try:
-        sevent.run(tcp_forward_servers, forward_servers, args.timeout,
-                   int(args.speed / 10), int(args.global_speed / 10))
+        main(sys.argv[1:])
+        sevent.instance().start()
     except KeyboardInterrupt:
-        for server, _ in forward_servers:
-            try:
-                server.close()
-            except: pass
         exit(0)
\ No newline at end of file
