@@ -1,9 +1,8 @@
 # -*- coding: utf-8 -*-
 
-import logging
 import socket
 import errno
-from .utils import is_py3
+from .utils import is_py3, get_logger
 from .event import EventEmitter, null_emit_callback
 from .loop import instance, MODE_IN, MODE_OUT
 from .buffer import Buffer, BaseBuffer, cbuffer, RECV_BUFFER_SIZE
@@ -50,6 +49,7 @@ class Socket(EventEmitter):
         self._is_enable_nodelay = False
         self._is_resolve = False
         self._has_drain_event = False
+        self.ignore_write_closed_error = False
 
         if self._socket:
             self._state = STATE_STREAMING
@@ -145,7 +145,7 @@ class Socket(EventEmitter):
             try:
                 self._socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
             except Exception as e:
-                logging.warning('nodela error: %s', e)
+                get_logger().warning('nodela error: %s', e)
                 self._is_enable_nodelay = False
                 return
         self._is_enable_nodelay = True
@@ -159,12 +159,12 @@ class Socket(EventEmitter):
             return
 
         if self._state in (STATE_INITIALIZED, STATE_CONNECTING):
-            self.close()
+            self._loop.add_async(self.close)
         else:
             if self._write_handler:
                 self._state = STATE_CLOSING
             else:
-                self.close()
+                self._loop.add_async(self.close)
 
     def close(self):
         if self._state == STATE_CLOSED:
@@ -174,20 +174,23 @@ class Socket(EventEmitter):
             try:
                 self._loop.remove_fd(self._fileno, self._connect_cb)
             except Exception as e:
-                logging.error("socket close remove_fd error:%s", e)
+                get_logger().error("socket close remove_fd error:%s", e)
             self._connect_handler = False
+            if self._connect_timeout_handler:
+                self._loop.cancel_timeout(self._connect_timeout_handler)
+                self._connect_timeout_handler = None
         elif self._state in (STATE_STREAMING, STATE_CLOSING):
             if self._read_handler:
                 try:
                     self._loop.remove_fd(self._fileno, self._read_cb)
                 except Exception as e:
-                    logging.error("socket close remove_fd error:%s", e)
+                    get_logger().error("socket close remove_fd error:%s", e)
                 self._read_handler = False
             if self._write_handler:
                 try:
                     self._loop.remove_fd(self._fileno, self._write_cb)
                 except Exception as e:
-                    logging.error("socket close remove_fd error:%s", e)
+                    get_logger().error("socket close remove_fd error:%s", e)
                 self._write_handler = False
 
         self._state = STATE_CLOSED
@@ -196,16 +199,16 @@ class Socket(EventEmitter):
                 try:
                     self._loop.clear_fd(self._fileno)
                 except Exception as e:
-                    logging.error("server close clear_fd error: %s", e)
+                    get_logger().error("server close clear_fd error: %s", e)
                 try:
                     self._socket.close()
                 except Exception as e:
-                    logging.error("socket close socket error:%s", e)
+                    get_logger().error("socket close socket error:%s", e)
 
             try:
                 self.emit_close(self)
             except Exception as e:
-                logging.exception("tcp emit close error:%s", e)
+                get_logger().exception("tcp emit close error:%s", e)
             self.remove_all_listeners()
             self._rbuffers.close()
             self._wbuffers.close()
@@ -217,7 +220,7 @@ class Socket(EventEmitter):
         self._loop.add_async(self.emit_error, self, error)
         self._loop.add_async(self.close)
         if self.emit_error == null_emit_callback:
-            logging.error("socket error:%s", error)
+            get_logger().error("socket error:%s", error)
 
     def _connect_cb(self):
         if self._state != STATE_CONNECTING:
@@ -257,6 +260,7 @@ class Socket(EventEmitter):
         self._connect_timeout = timeout
 
         def on_timeout_cb():
+            self._connect_timeout_handler = None
             if self._state == STATE_CONNECTING:
                 if not self._is_enable_fast_open:
                     self._error(ConnectTimeout("connect time out %s" % str(address)))
@@ -285,14 +289,14 @@ class Socket(EventEmitter):
                     try:
                         self._socket.setsockopt(socket.SOL_TCP, 23, 5)
                     except Exception as e:
-                        logging.warning('fast open error: %s', e)
+                        get_logger().warning('fast open error: %s', e)
                         self._is_enable_fast_open = False
 
                 if self._is_enable_nodelay:
                     try:
                         self._socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
                     except Exception as e:
-                        logging.warning('nodela error: %s', e)
+                        get_logger().warning('nodela error: %s', e)
                         self._is_enable_nodelay = False
 
                 if self._is_enable_fast_open:
@@ -445,7 +449,7 @@ class Socket(EventEmitter):
                         self._error(e)
                     return False
                 elif e.args[0] == errno.ENOTCONN:
-                    logging.error('fast open not supported on this OS')
+                    get_logger().error('fast open not supported on this OS')
                     self._is_enable_fast_open = False
                 self._error(e)
                 return False
@@ -522,6 +526,8 @@ class Socket(EventEmitter):
                 if self._is_enable_fast_open and self._is_resolve and not self._connect_handler:
                     return self._connect_and_write()
                 return False
+            if self.ignore_write_closed_error:
+                return False
             raise SocketClosed()
 
         if data.__class__ == Buffer:
@@ -546,13 +552,15 @@ class Socket(EventEmitter):
         return False
 
     def link(self, socket):
-        assert isinstance(socket, Socket), 'not Socket'
+        if self._state not in (STATE_STREAMING, STATE_CONNECTING):
+            raise SocketClosed()
+        if socket._state not in (STATE_STREAMING, STATE_CONNECTING):
+            raise SocketClosed()
 
+        self.ignore_write_closed_error = True
+        socket.ignore_write_closed_error = True
         rbuffer, wbuffer = socket.buffer
         if self._state != STATE_STREAMING:
-            if self._state != STATE_CONNECTING:
-                raise SocketClosed()
-
             if self._is_enable_fast_open and rbuffer:
                 self.write(rbuffer)
 
@@ -570,10 +578,7 @@ class Socket(EventEmitter):
             socket.on_data(lambda s, data: self.write(data))
 
         if socket._state != STATE_STREAMING:
-            if socket._state != STATE_CONNECTING:
-                raise SocketClosed()
-
-            if self._is_enable_fast_open and self._rbuffers:
+            if socket._is_enable_fast_open and self._rbuffers:
                 socket.write(self._rbuffers)
 
             def on_pconnect(s):
@@ -593,7 +598,7 @@ class Socket(EventEmitter):
 
 
 class Server(EventEmitter):
-    def __init__(self, loop=None, dns_resolver = None):
+    def __init__(self, loop=None, dns_resolver=None):
         EventEmitter.__init__(self)
         self._loop = loop or instance()
         self._dns_resolver = dns_resolver or DNSResolver.default()
@@ -609,6 +614,9 @@ class Server(EventEmitter):
     def __del__(self):
         self.close()
 
+    def on_listen(self, callback):
+        self.on("listen", callback)
+
     def on_connection(self, callback):
         self.on("connection", callback)
 
@@ -618,6 +626,9 @@ class Server(EventEmitter):
     def on_error(self, callback):
         self.on("error", callback)
 
+    def once_listen(self, callback):
+        self.on("listen", callback)
+
     def once_connection(self, callback):
         self.on("connection", callback)
 
@@ -673,19 +684,20 @@ class Server(EventEmitter):
                     try:
                         self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                     except Exception as e:
-                        logging.warning('reuseaddr error: %s', e)
+                        get_logger().warning('reuseaddr error: %s', e)
                         self._is_reuseaddr = False
 
                 if self._is_enable_fast_open:
                     try:
                         self._socket.setsockopt(socket.SOL_TCP, 23, 5)
                     except Exception as e:
-                        logging.warning('fast open error: %s', e)
+                        get_logger().warning('fast open error: %s', e)
                         self._is_enable_fast_open = False
 
                 self._socket.bind(addr[4])
                 self._accept_handler = self._loop.add_fd(self._fileno, MODE_IN, self._accept_cb)
                 self._socket.listen(backlog)
+                self._loop.add_async(self.emit_listen, self)
             except Exception as e:
                 self._loop.add_async(self._error, e)
 
@@ -705,7 +717,7 @@ class Server(EventEmitter):
     def _error(self, error):
         self._loop.add_async(self.emit_error, self, error)
         self.close()
-        logging.error("server error: %s", error)
+        get_logger().error("server error: %s", error)
 
     def close(self):
         if self._state in (STATE_INITIALIZED, STATE_LISTENING):
@@ -713,24 +725,24 @@ class Server(EventEmitter):
                 try:
                     self._loop.remove_fd(self._fileno, self._accept_cb)
                 except Exception as e:
-                    logging.error("server close remove_fd error: %s", e)
+                    get_logger().error("server close remove_fd error: %s", e)
                 self._accept_handler = False
             if self._socket is not None:
                 try:
                     self._loop.clear_fd(self._fileno)
                 except Exception as e:
-                    logging.error("server close clear_fd error: %s", e)
+                    get_logger().error("server close clear_fd error: %s", e)
                 try:
                     self._socket.close()
                 except Exception as e:
-                    logging.error("server close socket error: %s", e)
+                    get_logger().error("server close socket error: %s", e)
             self._state = STATE_CLOSED
 
             def on_close():
                 try:
                     self.emit_close(self)
                 except Exception as e:
-                    logging.exception("tcp server emit close error:%s", e)
+                    get_logger().exception("tcp server emit close error:%s", e)
                 self.remove_all_listeners()
             self._loop.add_async(on_close)
 
