@@ -3,6 +3,7 @@
 # create by: snower
 
 import sys
+import logging
 
 if sys.version_info[0] >= 3:
     is_py3 = True
@@ -34,4 +35,13 @@ def ensure_bytes(s):
 def ensure_unicode(s):
     if isinstance(s, byte_type):
         return s.decode("utf-8")
-    return s
\ No newline at end of file
+    return s
+
+_logger = logging
+
+def set_logger(logger):
+    global _logger
+    _logger = logger
+
+def get_logger():
+    return _logger
\ No newline at end of file
